#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <mqueue.h>
#include <sys/stat.h>
#include <sys/wait.h>
// Simple message wrapper.
struct message {
char mtext[128];
 };

 int send_msg(int qid, int pri, const char text[])
 {
 int r = mq_send(qid, text, strlen(text) + 1, pri);
 if (r == -1) {
 perror("mq_send");
 }
 return r;
 }

 void producer(mqd_t qid)
 {
 // Low priority messages
 send_msg(qid, 1, "This is my first message.");
 send_msg(qid, 1, "This is my second message.");

 // High priority message...
 send_msg(qid, 3, "No more messages.");
 }

 void consumer(mqd_t qid)
 {
 struct mq_attr mattr;

 // We assume the producer is finished at this point.
 do {
 u_int pri;
 struct message msg;
 ssize_t len;

 len = mq_receive(qid, (char *) &msg, sizeof(msg), &pri);
 if (len == -1) {
 perror("mq_receive");
 break;
 }
 printf("got pri %d '%s' len=%d\n", pri, msg.mtext, len);

 // Check for more messages in the queue.
 int r = mq_getattr(qid, &mattr);
 if (r == -1) {
 perror("mq_getattr");
 break;
 }
 } while (mattr.mq_curmsgs); // Stop when no more messages
 }
 int main(int argc, char *argv[])
 {
 // Allow up to 10 messages before blocking.
 // Message size is 128 bytes (see above).
 struct mq_attr mattr = {
 .mq_maxmsg = 10,
 .mq_msgsize = sizeof(struct message)
 };

 mqd_t mqid = mq_open("/myq",
 O_CREAT | O_RDWR,
 S_IREAD | S_IWRITE,
 &mattr);

 if (mqid == (mqd_t) -1) {
 perror("mq_open");
 exit(1);
 }

 // Fork a producer process, we'll be the consumer.
 pid_t pid = fork();
 if (pid == 0) {
 producer(mqid);
 mq_close(mqid);
 exit(0);
 }
 else {
 // Wait for the producer to send all messages so
 // we can illustrate priority.
 int status;
 wait(&status);

 consumer(mqid);
 mq_close(mqid);
 }

 mq_unlink("/myq");
 return 0;
 }
